class Battle_Other:
	def comp(self): #コンプリートしたとき
		import time
		print('全てのイズモンを獲得した!')
		time.sleep(1)
		return
	
	def set_turn(self):
		turn = 0
		return turn
		
	def get(self, ene : int): #獲得処理
		from ..Basic.Common_Function import Common_Function as co_f
		from ..Basic.Common_Variable import Common_Variable as co_v
		from ..Basic.Settings import Settings as st
		import random
		IZ_SET = st.IZUMON_SETTING
		prob = IZ_SET[ene][8] #獲得率
		if random.random() <= prob:
			before = sum(co_v.fellows) #獲"得前の数
			print(IZ_SET[ene][1], "を獲得した!")
			print("種族: %s HP: %d 攻撃: %d 防御: %d 特攻: %d 特防: %d" %
			(st.RACE[IZ_SET[ene][2]], IZ_SET[ene][3], IZ_SET[ene][4], IZ_SET[ene][5], IZ_SET[ene][6], IZ_SET[ene][7]))
			co_v.fellows[ene] = 1
			if sum(co_v.fellows) == st.IZ_NUM and before == st.IZ_NUM - 1: #初めてコンプリートした時のみ実行
				self.comp()
		else:
			co_f.show("逃げられた…")
		return
		
	def turn_change(self, turn : int): #ターンを切り替える
		if turn == 1:
			return 0
		else:
			return 1
		
	def change_izmon(self, party_num, typ):
		from ..Basic.Common_Function import Common_Function as co_f
		from ..Basic.Common_Variable import Common_Variable as co_v
		from .Battle_Caluculate import Battle_Caluculate as ba_c
		from ..Basic.Settings import Settings as st
		from ..Gym import Gym
		gym = Gym()

		print("~" * st.SEPALATE_LEN)
		print("パーティー一覧")
		if typ == 1:
			for i in range(st.PARTY_NUM):
				if co_v.party[i] != 0:
					print(i, ":", st.IZUMON_SETTING[co_v.party[i]][1], "HP:", gym.hp_me[i])
		else:
			for i in range(st.PARTY_NUM):
				if co_v.party[i] != 0:
					print(i, ":", st.IZUMON_SETTING[co_v.party[i]][1], "HP:", ba_c.set_hp(co_v.party[i], co_v.level[i]))
		print("~" * st.SEPALATE_LEN)
		me_sub = co_f.typing("どれを出す？", 1, party_num)
		sub = [me_sub, 0]
		return me_sub, sub
	
	def level_up(self, me):
		from ..Basic.Common_Function import Common_Function as co_f
		from ..Basic.Common_Variable import Common_Variable as co_v
		from ..Basic.Settings import Settings as st
		level_before = co_v.level[me]
		level_max = st.LEVEL_MAX[sum(co_v.badges)]
		co_v.num_killed += 1
		if level_before == level_max:
			co_f.show("レベルが上限じゃぞ")
			co_f.show("ジムでもっと鍛えるのじゃ")
		else:
			if co_v.num_killed == st.LEVELUP_NUM:
				co_f.show("レベルアップした!")
				co_v.level[me] += 1
				co_v.num_killed = 0
			else:
				print("倒した数")
				co_f.show(str(co_v.num_killed) + '/' + str(st.LEVELUP_NUM))
		return


class End:
	from ..Basic.Common_Function import Common_Function as co_f
	from ..Basic.Common_Variable import Common_Variable as co_v
	from ..Basic.Settings import Settings as st
	
	def save(self): #データをセーブする
		from ..Save_Code.Encode import Encode
		
		ty = self.co_f.typing("データをセーブしますか？ 1:する 0:しない", 1, 2)
		if ty == 1:
			code = Encode.encode()
			print("-" * self.st.SEPALATE_LEN)
			print("プレイヤー名と共に保管してください。")
			print("コード:", code)
			print("プレイヤー名:", self.co_v.player_name)
		print("=" * self.st.SEPALATE_LEN)
		return


class Set:
    from .Basic.Common_Function import Common_Function as co_f
    from .Basic.Common_Variable import Common_Variable as co_v
    from .Save_Code.Encode import Encode as encode
    from .Save_Code.Decode import Decode as decode

    from .Basic.Settings import Settings as st

    def main(self):
        print("=" * self.st.SEPALATE_LEN)
        while True:
            action = self.co_f.typing("何をしますか？ 0:データコード名前変更 1:表示秒数変更 2:プレイヤー名変更 3:やめる", 1, 4)
            if action == 0:
                self.change_name_code()
            elif action == 1:
                self.change_show_length()
            elif action == 2:
                self.change_name_v()
            else:
                break
            print("-" * self.st.SEPALATE_LEN)
        print("=" * self.st.SEPALATE_LEN)
        return
    
    def change_name_v(self):
        self.co_v.player_name = self.co_f.typing("新しいプレイヤー名", 0)
        print("変更しました。")

    def change_name_code(self):
        tmp_fellow = self.co_v.fellows
        tmp_party = self.co_v.party
        tmp_level = self.co_v.level
        tmp_badges = self.co_v.badges
        tmp_name = self.co_v.player_name
        self.co_f.show("データコードに保存されているプレイヤー名を変更します。", 2) 
        code = self.co_f.typing("データコード", 0)
        self.co_v.player_name = self.co_f.typing("名前", 0)
        self.decode(code)
        code_new = self.encode()
        self.co_v.fellows = tmp_fellow
        self.co_v.party = tmp_party
        self.co_v.level= tmp_level
        self.co_v.badges = tmp_badges
        self.co_v.player_name = tmp_name
        del tmp_badges, tmp_fellow, tmp_level, tmp_party, tmp_name
        print("変更しました。")
        print("新しいコード：", code_new)
        return
    
    def change_show_length(self):
        length = self.co_f.typing("表示秒数を入力してください(最長10秒)", 1, 10)
        self.co_v.show_length = length
        print("設定しました。")
        return



class Battle_Caluculate:
	def calculate_damage(damage_all:list, tech:list, me:int, ene:int, i:int, ene_level:int):
		from ..Basic.Common_Variable import Common_Variable as co_v
		from ..Basic.Settings import Settings as st
		from .Battle_Set import Battle_Set as ba_s
		IZ_SET = st.IZUMON_SETTING; IZ_FIG = st.SETS_FIGHT
		num = [me, ene]
		level = [co_v.level[me], ene_level]
		for j in range(2):
			me_kind = IZ_SET[num[j]][2]
			ene_kind = IZ_SET[num[j - 1]][2]
			attack_normal = (IZ_SET[num[j]][4] * 2 + 30) * level[j] / 100
			attack_spetial = (IZ_SET[num[j]][6] * 2 + 30) * level[j] / 100
			protect_normal = (IZ_SET[num[j - 1]][5] * 2 + 30) * level[j - 1] / 100
			protect_spetial = (IZ_SET[num[j - 1]][7] * 2 + 30) * level[j - 1] / 100
			tech_nomal = (tech[4][i] * 2 + 30) * level[j] / 100
			tech_spetial = (tech[6][i] * 2 + 30) * level[j] / 100
			damage_nomal = ((22 * tech_nomal * attack_normal / protect_normal) * 0.02 + 2) * IZ_FIG[me_kind][ene_kind]
			damage_spe = ((22 * tech_spetial * attack_spetial / protect_spetial) * 0.02 + 2) * IZ_FIG[me_kind][ene_kind]
			damage_all[j][i] = int(damage_nomal + 0.5)
			damage_all[2 + j][i] = int(damage_spe + 0.5)
			expect = (st.ESCAPE_P * 2 + 30) * level[j]
			tech[8 + j][i] = damage_nomal / expect # 命中率を設定
			tech[10 + j][i] = damage_spe / expect

		return tech, damage_all
	
	def store_tech(tech : list, me : int, ene : int, i : int):
		from ..Basic.Common_Variable import Common_Variable as co_v
		from ..Basic.Settings import Settings as st
		IZ_SET = st.IZUMON_SETTING; TH_NOM = st.TECH_NOMAL; TH_SPE = st.TECH_SPETIAL
		TH_IND = st.TECH_INDIVIDUAL
		
		kind = [IZ_SET[me][2], IZ_SET[ene][2]]
		num = [me, ene]
		for j in range(2):
			if IZ_SET[num[j]][9] == 0:
				tech[j][i] = TH_NOM[0][kind[j]] #技名を格納
				tech[2 + j][i] = TH_SPE[0][kind[j]]
				tech[4 + j][i] = TH_NOM[1][kind[j]]
				tech[6 + j][i] = TH_SPE[1][kind[j]]
			else:
				num_tech = IZ_SET[num[j]][9] - 1 #技番号
				tech[j][i] = TH_IND[0][num_tech]
				tech[2 + j][i] = TH_IND[1][num_tech]
				tech[4 + j][i] = TH_IND[2][num_tech]
				tech[6 + j][i] = TH_IND[3][num_tech]
			
			if "%" in tech[0][i]: #"%"のユーザー名を置き換える
				tech[j][i] = tech[0 + j][i].replace("%", co_v.player_name)
			if "%" in tech[2][i]:
				tech[2 + j][i] = tech[2 + j][i].replace("%", co_v.player_name)
		return tech
	
	def set_hp(izmon : int, level : int):
		from ..Basic.Settings import Settings as st
		hp = (st.IZUMON_SETTING[izmon][3] * 2 + 30) * level / 100 + level + 10
		return int(hp)
